/*
myseller.pl

A Prolog program that simulates a seller, and can
be used in the 'negotiation game'.  The seller is
called with the predicate seller(Cost, DTS, History, Offer)
where:
  -Cost is the cost of the item to sell.
  -DTS is the days left for the seller to negotiate.
  -History is a list detailing the history of the
   negotiation so far, with the seller always going first.
  -Offer is the new offer that the seller arrives at.

Author    : Gareth Jones.
Date      : 20.04.02.
Reference : CC283 Assignment 2.
*/

/*
This procedure provides the seller with the
offers made by the buyer so far.
*/
buyer_offers([], []).
buyer_offers([_], []).
buyer_offers([X, Y|T], [X1|T1]):-
  X1 is abs(X),
  buyer_offers(T, T1).

/*
This procedure provides the seller with the
number of offers made by the buyer so far.
*/
num_offers([], 0).
num_offers([_|T], Result):-
  num_offers(T, Subresult),
  Result is 1 + Subresult.

/*
Special case - seller has only this one time to
set the offer, and so goes for a 10% profit.
*/
seller(Cost, 0, [], Offer):-
  Offer is floor(Cost + (Cost / 10)),
  seller_writeln(['Seller going for 10% profit: ', Offer]).

/*
Seller's initial offer aims for a 100% profit.
*/
seller(Cost, _, [], Offer):-
  Offer is 2 * Cost,
  seller_writeln(['Seller starts by asking for 100% profit: ', Offer]).

/*
Seller goes for a 10% profit, or accepts the buyer's
last bid (whichever is larger) when out of time.
*/
seller(Cost, 0, [Bid|_], Offer):-
  MinProfit is floor(Cost + (Cost / 10)),
  (
    MinProfit > abs(Bid) -> Offer is MinProfit;
                            Offer is abs(Bid)
  ),
  seller_writeln(['Final offer of seller: ', Offer]).

/*
With not much time left (3 days or less), the seller
will try to nudge the offer up slightly (5% of Cost) whilst
ensuring that no offer made makes less than a 10% profit.
*/
seller(Cost, DTS, [LB|_], Offer):-
  DTS < 4,
  LastBid is abs(LB),
  MinProfit is floor(Cost + (Cost / 10)),
  SmallNudge is floor(Cost / 20),
  (   
    (
      LastBid < MinProfit,
      Offer is MinProfit,
      seller_writeln(['Seller refuses bid. Insists on: ', Offer])
    );
    (
      Offer is LastBid + SmallNudge,
      seller_writeln(['Seller asks buyer for more: ', Offer])
    )
  ).

/*
When the seller has more than 3 days left to negotiate
with the buyer...
*/
seller(Cost, DTS, History, Offer):-
  DTS > 3,
/*
  Get buyer's history and calculate how many days have passed.
  This information is then used to calculate the number of days
  that the seller has to adjust its offer to the buyer.
*/
  buyer_offers(History, BuyerHistory),
  num_offers(BuyerHistory, DaysGone),
  DaysToReduce is (DTS + DaysGone) - 3,
/*
  A reduction value is found by:
  -Halving the product cost.
  -Dividing this value by DaysToReduce.
*/
  Reduction is floor((Cost / 2) / DaysToReduce),
/*
  A tentative new offer is created by starting with the
  offer set for a 100% profit.  This figure is then
  reduced by the amount:
    Reduction * (number of days passed + 1 (current day))
*/
  TestOffer is (2 * Cost) - ((DaysGone + 1) * Reduction),
  BuyerHistory = [Bid|_],
  (
/*
      If the buyer's latest bid is higher than the
      tentative bid...
*/
    (
      Bid > TestOffer,
      Offer is floor(Bid + (Reduction / 3)),
      seller_writeln(['Seller asks buyer for more: ', Offer])
    );
/*
      Else if the tentative bid is greater than (but
      not by too much) the buyer's latest bid...
*/
    (	
      TestOffer >= Bid,
      TestOffer =< (Bid + Reduction),
      Offer is TestOffer,
      seller_writeln(['Seller insists on: ', Offer])
    );
/*
      Else reduce the tentative offer by a little
      and use that as the offer.
*/
    (
      Offer is TestOffer - Reduction,
      seller_writeln(['Seller insists on: ', Offer])
    )
  ).

/***********************************************************/

seller_writeln(_) :- mediator_parameter(no_explanation), !.
	/* no output if mediator suppresses reasoning */
seller_writeln([]) :- nl, !.
seller_writeln([H|T]) :- write(H), seller_writeln(T), !.

/***********************************************************/
